Package: conMItion
Type: Package
Title: Conditional Mutual Information Estimation for Multi-Omics Data
Version: 0.4.0
Authors@R: c(
    person(
      "Gaojianyong", "Wang",
      email = "gjywang@gmail.com",
      role = c("aut", "cre")
    )
  )
Description: The biases introduced in association measures, particularly mutual information, 
    are influenced by factors such as tumor purity, mutation burden, and hypermethylation. 
    This package provides the estimation of conditional mutual information (CMI) and its 
    statistical significance with a focus on its application to multi-omics data. Utilizing 
    B-spline functions (inspired by Daub et al. (2004) <doi:10.1186/1471-2105-5-118>), the package offers tools to estimate the association between heterogeneous multi-
    omics data, while removing the effects of confounding factors. This helps to unravel complex
    biological interactions. In addition, it includes methods to evaluate the statistical significance 
    of these associations, providing a robust framework for multi-omics data integration and 
    analysis. This package is ideal for researchers in computational biology, bioinformatics, 
    and systems biology seeking a comprehensive tool for understanding interdependencies in 
    omics data.
License: GPL-2
Encoding: UTF-8
RoxygenNote: 7.3.3
NeedsCompilation: yes
Packaged: 2026-06-17 14:34:15 UTC; gwang
Author: Gaojianyong Wang [aut, cre]
Maintainer: Gaojianyong Wang <gjywang@gmail.com>
Repository: CRAN
Date/Publication: 2026-06-17 15:00:02 UTC
Built: R 4.5.2; x86_64-apple-darwin20; 2026-06-17 18:49:32 UTC; unix
Archs: conMItion.so.dSYM
