# Generated by roxygen2: do not edit by hand

export(CMIBiCondimat2mat)
export(CMIBiCondimat2matAll)
export(CMIBiCondimat2matPermu)
export(CMIBiCondimat2vec)
export(CMIBiCondimat2vecPermu)
export(CMImat2mat)
export(CMImat2matAll)
export(CMImat2matPermu)
export(CMImat2vec)
export(CMImat2vecPermu)
export(CORmat2vecPermu)
export(MImat2mat)
export(MImat2matAll)
export(MImat2matPermu)
export(MImat2vec)
export(MImat2vecPermu)
export(getCMI)
export(getCMIBiCondi)
export(getEntropy)
export(getEntropyBi)
export(getEntropyQuadri)
export(getEntropyTri)
export(getMI)
export(getMIBi)
export(getPValue)
importFrom(stats,cor)
importFrom(stats,runif)
useDynLib(conMItion)
